#ifndef FAR_MANAGER_INLINE_INCLUDED
#define FAR_MANAGER_INLINE_INCLUDED

#include <cstring>

#include "plugin.hpp"
#include "countof.h"

inline
FarMenuItem
menuItem(const char* title, bool selected, bool checked)
{
	FarMenuItem result = {
		"",
		selected ? 1 : 0,
		checked ? 1 : 0,
		0
	};
	std::strncpy(result.Text, title, COUNTOF(result.Text));
	return result;
}

inline
EnableIf::EnableIf(bool condition)
: condition_(condition)
{
}

inline
void
EnableIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? 0 : MIF_DISABLE;
}

inline
CheckIf::CheckIf(bool condition)
: condition_(condition)
{
}

inline
void
CheckIf::operator()(FarMenuItem& item) const
{
	item.Checked = condition_ ? 1 : 0;
}

inline
void
CheckIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? MIF_CHECKED : 0;
}

inline
SelectIf::SelectIf(bool condition)
: condition_(condition)
{
}

inline
void
SelectIf::operator()(FarMenuItem& item) const
{
	item.Checked = condition_ ? 7 : 0;
	item.Selected = condition_ ? 1 : 0;
}

inline
void
SelectIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? (MIF_SELECTED | 7) : 0;
}

inline
FarMenuItems&
FarMenuItems::operator()(const char* title)
{
	resize(size() + 1);
	std::strncpy(back().Text, title, COUNTOF(back().Text));
	back().Selected = 0;
	back().Checked = 0;
	back().Separator = 0;
	return *this;
}

inline
FarMenuItems&
FarMenuItems::operator()(LanguageString title)
{
	return (*this)(title.get());
}

inline
FarMenuItems&
FarMenuItems::operator()()
{
	push_back(menuSeparator);
	return *this;
}

template <typename Functor>
FarMenuItems&
FarMenuItems::operator()(const Functor& functor)
{
	functor(back());
	return *this;
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()(const char* title)
{
	resize(size() + 1);
	back().Flags = 0;
	std::strncpy(&back().Text.Text[0], title, COUNTOF(back().Text.Text));
	back().AccelKey = 0;
	back().Reserved = 0;
	back().UserData = 0;
	return *this;
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()(LanguageString title)
{
	return (*this)(title.get());
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()()
{
	resize(size() + 1);
	back().Flags = MIF_SEPARATOR;
	back().Text.Text[0] = 0;
	return *this;
}

template <typename Functor>
FarMenuItemsEx&
FarMenuItemsEx::operator()(const Functor& functor)
{
	functor(back());
	return *this;
}

inline
LanguageString::LanguageString(int id)
: id_(id)
{
}

inline
LanguageString::operator const char*() const
{
	return get();
}

inline
const char* LanguageString::get() const
{
	return farManager.getMsg(id_);
}

inline
FarManager&
FarManager::operator=(const PluginStartupInfo& other)
{
	PluginStartupInfo::operator=(other);
	return *this;
}

inline
const char*
FarManager::moduleName() const
{
	return ModuleName;
}

inline
int
FarManager::editorControl(int command, void* param)
{
	return EditorControl(command, param);
}

inline
int
FarManager::inputBox(const char* title, const char* prompt, const char* history,
	const char* srcText, char* destText, int destLength,
	const char* help,
	DWORD flags)
{
	return InputBox(title, prompt, history,
		srcText, destText, destLength,
		help,
		flags);
}

inline
int
FarManager::menu(const FarMenuItem* items, size_t itemCount,
	const char* title,
	const char* bottom,
	const char* help,
	const int* breakKeys,
	int* breakCode)
{
	return Menu(ModuleNumber,
		-1, -1, 0,
		FMENU_WRAPMODE,
		title,
		bottom,
		help,
		breakKeys, breakCode,
		items, itemCount);
}

inline
int
FarManager::menu(const FarMenuItems& items,
	const char* title,
	const char* bottom,
	const char* help,
	const int* breakKeys,
	int* breakCode)
{
	return menu(&items[0], items.size(), title, bottom, help, breakKeys, breakCode);
}

inline
int
FarManager::menu(const FarMenuItemEx* items, size_t itemCount,
	const char* title,
	const char* bottom,
	const char* help,
	const int* breakKeys,
	int* breakCode)
{
	return Menu(ModuleNumber,
		-1, -1, 0,
		FMENU_WRAPMODE | FMENU_USEEXT,
		title,
		bottom,
		help,
		breakKeys, breakCode,
		reinterpret_cast<const FarMenuItem*>(items), itemCount);
}

inline
int
FarManager::menu(const FarMenuItemsEx& items,
	const char* title,
	const char* bottom,
	const char* help,
	const int* breakKeys,
	int* breakCode)
{
	return menu(&items[0], items.size(), title, bottom, help, breakKeys, breakCode);
}

inline
int
FarManager::dialog(FarDialogItem* items, size_t itemCount,
	int width, int height,
	const char* help)
{
	return Dialog(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount);
}

inline
int
FarManager::dialogEx(FarDialogItem* items, size_t itemCount,
	int width, int height,
	const char* help,
	FARWINDOWPROC dlgProc)
{
	return DialogEx(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount,
		0, // Reserved
		0, // Flags
		dlgProc,
		0);
}


inline
long
FarManager::setTextPtr(HANDLE hDlg, int itemId, const char* text)
{
	return SendDlgMessage(hDlg, DM_SETTEXTPTR, itemId, reinterpret_cast<long>(text));
}

inline
long
FarManager::defDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2)
{
	return DefDlgProc(hDlg, Msg, Param1, Param2);
}

inline
const char*
FarManager::getMsg(int id) const
{
	return GetMsg(ModuleNumber, id);
}

inline
void
FarManager::showMessage(const char* text, const char* title) const
{
	const char* items[2];
	items[0] = title;
	items[1] = text;
	Message(ModuleNumber,
		FMSG_MB_OK, 0, items, 2, 0);
}

inline
void
FarManager::showMessage(const std::string& s, const char* title) const
{
	return showMessage(s.c_str(), title);
}

inline
void
FarManager::showMessage(const std::ostringstream& oss, const char* title) const
{
	return showMessage(oss.str().c_str(), title);
}

inline
void
FarManager::showError(const char* text) const
{
	const char* items[2];
	items[0] = "Error";
	items[1] = text;
	Message(ModuleNumber,
		FMSG_WARNING | FMSG_MB_OK, 0, items, 2, 0);
}

inline
void
FarManager::showError(const std::exception& e) const
{
	showError(e.what());
}

#endif